@@ -31,7 +31,7 @@ except ImportError:
 # The Onionprobe version string
 # Uses Semantic Versioning 2.0.0
 # See https://semver.org
-onionprobe_version = '0.3.2'
+onionprobe_version = '1.1.0'
 
 # The base path for this project
 basepath = os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir, os.pardir) + os.sep
@@ -69,11 +69,26 @@ config = {
             },
 
         'control_password': {
-            'help'    : 'Set Tor control password or use a password prompt (system-wide Tor service) or auto-generate a temporary password (built-in Tor service)',
+            'help'    : """Set Tor control password or use a password prompt
+                           (system-wide Tor service) or auto-generate a temporary password
+                           (built-in Tor service)""".strip(),
             'default' : None,
             'action'  : 'store',
             },
 
+        'metrics_port': {
+            'help'    : 'Tor Metrics port (MetricsPort). An empty value means it is disabled',
+            'default' : '',
+            'action'  : 'store',
+            },
+
+        'metrics_port_policy': {
+            'help'    : """Tor Metrics port policy (MetricsPortPolicy).
+                           An empty value means it is disabled'""".strip(),
+            'default' : 'reject *',
+            'action'  : 'store',
+            },
+
         'loop': {
             'help'    : 'Run Onionprobe continuously',
             'default' : False,
@@ -88,7 +103,7 @@ config = {
 
         'prometheus_exporter_port': {
             'help'    : 'Set the Prometheus exporter port',
-            'default' : 9091,
+            'default' : 9935,
             'action'  : 'store',
             },
 
@@ -110,6 +125,25 @@ config = {
             'action'  : argparse.BooleanOptionalAction,
             },
 
+        'tls_verify': {
+            'help'    : 'Whether to verify TLS/HTTPS certificates',
+            'default' : True,
+            'action'  : argparse.BooleanOptionalAction,
+            },
+
+        'test_tls_connection': {
+            'help'    : 'Whether to run a specific test for TLS endpoints',
+            'default' : True,
+            'action'  : argparse.BooleanOptionalAction,
+            },
+
+        'get_certificate_info': {
+            'help'    : """Whether to get certificate information when testing TLS/HTTPS endpoints.
+                           Requires --test_tls_connection to take effect.""".strip(),
+            'default' : True,
+            'action'  : argparse.BooleanOptionalAction,
+            },
+
         'interval': {
             'help'    : 'Max random interval in seconds between probing each endpoint',
             'default' : 60,
@@ -123,7 +157,10 @@ config = {
             },
 
         'rounds': {
-            'help'    : 'Run only a limited number of rounds (i.e., the number of times that Onionprobe tests all the configured endpoints). Requires the "loop" option to be enabled. Set to 0 to disable this limit.',
+            'help'    : """Run only a limited number of rounds (i.e., the
+                           number of times that Onionprobe tests all the configured
+                           endpoints). Requires the "loop" option to be enabled. Set to 0 to
+                           disable this limit.""".strip(),
             'default' : 0,
             'action'  : 'store',
             },
@@ -140,6 +177,12 @@ config = {
             'action'  : 'store',
             },
 
+        'tls_connect_timeout': {
+            'help'    : 'Connection timeout for TLS connections',
+            'default' : 30,
+            'action'  : 'store',
+            },
+
         'http_connect_timeout': {
             'help'    : 'Connection timeout for HTTP/HTTPS requests',
             'default' : 30,
@@ -158,6 +201,12 @@ config = {
             'action'  : 'store',
             },
 
+        'tls_connect_max_retries': {
+            'help'    : 'Max retries when doing a TLS connection to an Onion Service',
+            'default' : 3,
+            'action'  : 'store',
+            },
+
         'circuit_stream_timeout': {
             'help'    : 'Sets how many seconds until a stream is detached from a circuit and try a new circuit',
             'default' : 60,
@@ -173,8 +222,9 @@ config = {
                     'port'    : '80',
                     'paths'   : [
                         {
-                            'path'   : '/',
-                            'pattern': 'Tor Project',
+                            'path'            : '/',
+                            'pattern'         : 'Tor Project',
+                            'allowed_statuses': [ 200 ],
                             },
                         ],
                     },
@@ -229,6 +279,9 @@ def cmdline_parser():
         else:
             config[argument]['type'] = type(config[argument]['default'])
 
+            if not isinstance(config[argument]['default'], bool) and config[argument]['default'] != '':
+                config[argument]['help'] += ' (default: %(default)s)'
+
             parser.add_argument('--' + argument, **config[argument])
 
     return parser
@@ -289,7 +342,7 @@ class OnionprobeConfig:
 class OnionprobeConfigCompiler:
     """Base class to build Onionprobe configs from external sources of Onion Services"""
 
-    def __init__(self, databases, config_template = None, output_folder = None):
+    def __init__(self, databases, config_template, output_folder, wait=0, config_overrides=None, loop=False):
         """
         Constructor for the OnionprobeConfigCompiler class.
 
@@ -308,41 +361,61 @@ class OnionprobeConfigCompiler:
         :param output_folder: Output folder where configs are written
         """
 
+        # Initialize the configuration object
+        self.config = {}
+
         # Save the databases of Onion Services
         self.databases = databases
 
-        # Determine the default configuration template
-        if config_template is None:
-            # Fallback to configs/ folder when running directly from the
-            # Onionprobe repository or from the python package
-            config_template = os.path.normpath(os.path.join(basepath, 'configs', 'tor.yaml'))
-
-            # Finally fallback to /etc/onionprobe
-            if not os.path.exists(config_template):
-                config_template = os.path.normpath(os.path.join(os.sep, 'etc', 'onionprobe', 'tor.yaml'))
-
         # Load the default configuration file as a template
         if os.path.exists(config_template):
             print('Loading configuration template from %s...' % (config_template))
 
-            with open(config_template, 'r') as config:
-                self.config = yaml.load(config, yaml.CLoader)
+            with open(config_template, 'r') as base_config:
+                self.config = yaml.load(base_config, yaml.CLoader)
 
         else:
             raise FileNotFoundError(config_template)
 
-        # Determine the output path
-        if output_folder is None:
-            # Fallback to configs/ folder when running directly from the
-            # Onionprobe repository
-            self.output_folder = os.path.join(basepath, 'configs')
+        # Set the output folder
+        self.output_folder = output_folder
 
-            # Fallback to the current working directory
-            if not os.path.exists(self.output_folder):
-                self.output_folder = os.getcwd()
+        if not os.path.exists(output_folder):
+            raise FileNotFoundError(output_folder)
 
-        else:
-            self.output_folder = output_folder
+        # Set wait time
+        self.wait = wait
+
+        # Set loop configuration
+        self.loop = loop
+
+        # Apply overrides
+        if config_overrides != None:
+            # Copy item by item, ensuring type casting
+            for item in config_overrides:
+                override = item.split('=')
+
+                if len(override) != 2:
+                    print('Skipping malformed override param %s...' % (item))
+                    continue
+
+                key   = str(override[0])
+                value = override[1]
+
+                if key not in config:
+                    print('Skipping unknown parameter %s...' % (key))
+                    continue
+
+                cast = type(config[key]['default'])
+
+                if cast == bool:
+                    value.lower()
+
+                    self.config[key] = True if value == 'true' else False
+                else:
+                    self.config[key] = cast(value)
+
+                print('Setting %s to %s.' % (key, value))
 
     def build_endpoints_config(self, database):
         """
@@ -390,10 +463,10 @@ class OnionprobeConfigCompiler:
                 endpoints = self.build_endpoints_config(database)
 
                 # Create a new config using the default as base
-                config = dict(self.config)
+                new_config = dict(self.config)
 
                 # Replace the endpoints
-                config['endpoints'] = endpoints
+                new_config['endpoints'] = endpoints
 
                 # Build the output path
                 output_folder = os.path.normpath(os.path.join(self.output_folder, database + '.yaml'))
@@ -402,11 +475,36 @@ class OnionprobeConfigCompiler:
                 with open(output_folder, 'w') as output:
                     print('Saving the generated config for database %s into %s...' % (database, output_folder))
 
-                    output.write(yaml.dump(config))
+                    output.write(yaml.dump(new_config))
 
             except Exception as e:
                 print(e)
 
+    def build_and_wait(self):
+        """
+        Build Onionprobe configs, then wait.
+        """
+
+        self.build_onionprobe_config()
+
+        if self.wait != 0:
+            import time
+
+            print('Waiting %s seconds...' % (self.wait))
+            time.sleep(self.wait)
+
+    def compile(self):
+        """
+        Main compilation procedure.
+
+        """
+
+        if self.loop is True:
+            while True:
+                self.build_and_wait()
+        else:
+            self.build_and_wait()
+
 def cmdline_parser_compiler(default_source=None):
     """
     Generate command line arguments for the configuration compiler
@@ -421,9 +519,59 @@ def cmdline_parser_compiler(default_source=None):
                     formatter_class=argparse.RawDescriptionHelpFormatter,
                   )
 
-    parser.add_argument('-s', '--source',          dest='source',          help="Database source file or endpoint. Defaults to " + default_source + '.')
-    parser.add_argument('-t', '--config_template', dest='config_template', help="Configuration template to use")
-    parser.add_argument('-o', '--output_folder',   dest='output_folder',   help="Output folder where config should be saved")
+    # Try to use the configs/ folder as the default config_template (will match
+    # when running directly from the Onionprobe repository or from the python
+    # package)
+    config_template = os.path.normpath(os.path.join(basepath, 'configs', 'tor.yaml'))
+
+    # Fallback config_template to /etc/onionprobe
+    if not os.path.exists(config_template):
+        config_template = os.path.normpath(os.path.join(os.sep, 'etc', 'onionprobe', 'tor.yaml'))
+
+    # Try to use the configs/ folder as the default output_folder (will match
+    # when running directly from the Onionprobe repository or from the python
+    # package)
+    output_folder = os.path.join(basepath, 'configs')
+
+    # Fallback output_folder to the current working directory
+    if not os.path.exists(output_folder):
+        output_folder = os.getcwd()
+
+    parser.add_argument('-s', '--source',
+            dest='source',
+            default=default_source,
+            help="Database source file or endpoint (default: %(default)s)")
+
+    parser.add_argument('-t', '--config_template',
+            dest='config_template',
+            default=config_template,
+            help="Configuration template to use (default %(default)s)")
+
+    parser.add_argument('-o', '--output_folder',
+            dest='output_folder',
+            default=output_folder,
+            help="Output folder where config should be saved (default: current working directory)")
+
+    parser.add_argument('-w', '--wait',
+            dest='wait',
+            default=0,
+            type=int,
+            help="""Wait a number of seconds before exiting after writing the config.
+                    Useful for a configurator container service tha should run periodically
+                    (default: %(default)s)""".strip())
+
+    parser.add_argument('-l', '--loop',
+            dest='loop',
+            action='store_true',
+            default=False,
+            help="""Whether to continuously generate configuration.
+                    Useful when set in conjunction with --wait (default: %(default)s)""")
+
+    parser.add_argument('-c', '--config_overrides',
+            dest='config_overrides',
+            default=None,
+            nargs='*',
+            help="Override configuration parameters in the form of param1=value1 ... paramN=valueN")
 
     return parser
 
